use serde::Deserialize;
use std::fs::File;
use std::io::Read;
use std::sync::OnceLock;

#[derive(Clone, Debug, Deserialize)]
pub struct Config {
    pub server: Server,
    pub logger: Logger,
    pub database: Database,
    pub jwt: Jwt,
    pub social: Social,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Server {
    pub name: String,
    pub port: u32,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Database {
    pub url: String,
    pub options: Options,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Options {
    pub min_conns: u32,
    pub max_conns: u32,
    pub conn_timeout: u64,
    pub idle_timeout: u64,
    pub max_lifetime: u64,
    pub sql_logging: bool,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Logger {
    pub dir: String,
    pub prefix: String,
    pub level: String,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Jwt {
    pub token_secret: String,
    pub refresh_token_secret: String,
    pub expires: i64,
    pub refresh_expires: i64,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Social {
    pub wechat: Wechat,
    // pub google: google::GoogleSocial,
    // pub facebook: facebook::FacebookSocial,
    // pub apple: apple::AppleSocial,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Wechat {
    pub app_id: String,
    pub app_secret: String,
}

#[macro_export]
macro_rules! config {
    () => {
        library::core::config::config()
    };
}

const CFG_FILE: &str = "app.toml";

static CFG: OnceLock<Config> = OnceLock::new();

impl Default for Config {
    fn default() -> Self {
        let mut file = match File::open(CFG_FILE) {
            Ok(f) => f,
            Err(err) => panic!("读取配置文件失败：{}", err),
        };
        let mut config_str = String::new();
        match file.read_to_string(&mut config_str) {
            Ok(s) => s,
            Err(err) => panic!("读取配置文件内容失败：{}", err),
        };
        toml::from_str(&*config_str).expect("格式化配置数据失败")
    }
}

pub fn config() -> &'static Config {
    CFG.get_or_init(|| Config::default())
}
