use crate::model::response::ResErr;
use axum::body::Body;
use axum::extract::Request;
use axum::http::header::CONTENT_TYPE;
use axum::http::HeaderMap;
use axum::middleware::Next;
use axum::response::{IntoResponse, Response};
use http_body_util::BodyExt;
use std::collections::HashMap;

/// 请求日志中间件
pub async fn handle(request: Request, next: Next) -> Response {
    let enter_time = chrono::Local::now();
    let req_method = request.method().to_string();
    let req_uri = request.uri().to_string();
    let req_header = header_to_string(request.headers());
    // let identity = match request.extensions().get::<Identity>() {
    //     Some(v) => v.to_string(),
    //     None => String::from("<none>"),
    // };

    let (response, body) = match drain_body(request, next).await {
        Err(err) => return err.into_response(),
        Ok(v) => v,
    };

    let duration = chrono::Local::now()
        .signed_duration_since(enter_time)
        .to_string();

    tracing::info!(
        method = req_method,
        uri = req_uri,
        headers = req_header,
        // identity = identity,
        body = body,
        duration = duration,
        "请求记录"
    );

    response
}

fn header_to_string(h: &HeaderMap) -> String {
    let mut map: HashMap<String, Vec<String>> = HashMap::new();

    for k in h.keys() {
        let mut vals: Vec<String> = Vec::new();

        for v in h.get_all(k) {
            if let Ok(s) = v.to_str() {
                vals.push(s.to_string())
            }
        }

        map.insert(k.to_string(), vals);
    }

    match serde_json::to_string(&map) {
        Ok(v) => v,
        Err(_) => String::from("<none>"),
    }
}

async fn drain_body(request: Request, next: Next) -> Result<(Response, Option<String>), ResErr> {
    let ok = match request
        .headers()
        .get(CONTENT_TYPE)
        .and_then(|value| value.to_str().ok())
    {
        Some(v) => {
            if v.starts_with("application/json")
                || v.starts_with("application/x-www-form-urlencoded")
            {
                true
            } else {
                false
            }
        }
        None => false,
    };

    if !ok {
        return Ok((next.run(request).await, None));
    }

    let (parts, body) = request.into_parts();

    // this wont work if the body is an long running stream
    let bytes = match body.collect().await {
        Ok(v) => v.to_bytes(),
        Err(err) => {
            tracing::error!(error = ?err, "err parse request body");
            return Err(ResErr::default());
        }
    };

    let body = std::str::from_utf8(&bytes)
        .and_then(|s| Ok(s.to_string()))
        .ok();

    let response = next
        .run(Request::from_parts(parts, Body::from(bytes)))
        .await;

    Ok((response, body))
}
